#!/usr/bin/env python

#****************************************************************************
# genboolean.py, provides a class for boolean formating
#
# Copyright (C) 2003, Douglas W. Bell
#
# This is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License, Version 2.  This program is
# distributed in the hope that it will be useful, but WITTHOUT ANY WARRANTY.
#*****************************************************************************

import re, types

_formatDict = {1:1, 0:0, '1':1, '0':0, 't':1, 'f':0, 'true':1, 'false':0, \
               'y':1, 'n':0, 'yes':1, 'no':0}

class GenBoolean:
    """Stores & formats boolean values"""
    def __init__(self, bool=0):
        """Accepts any format from _formatDict (not case sensitive)
           to initialize"""
        self.setBool(bool)

    def setBool(self, bool):
        """Sets the value from any format in _formatDict
           (not case sensitive)"""
        if type(bool) in (types.StringType, types.UnicodeType):
            bool = bool.lower()
        try:
            self.bool = _formatDict[bool]
        except KeyError:
            raise GenBooleanError('Invalid entry')

    def setFromFormat(self, boolStr, format):
        """Set value based on given format only
           Returns self"""
        try:
            self.bool = not list(_splitFormat(format)).index(boolStr.lower())
        except ValueError:
            raise GenBooleanError('Invalid entry, no format match')
        return self

    def boolStr(self, format='true/false'):
        """Return string based on the format, which includes a true
           string and a false string separated by a '/'
           Raise exception of format invalid"""
        return _splitFormat(format)[not self.bool]

    def clone(self):
        """Return cloned instance"""
        return self.__class__(self.bool)

    def __repr__(self):
        """Outputs in format true/false"""
        return self.boolStr()

    def __cmp__(self, other):
        """Compare operator"""
        if other and isinstance(other, GenBoolean):
            return cmp(self.bool, other.bool)
        if type(other) is types.IntType:
            return cmp(self.bool, other)
        return 1

    def __hash__(self):
        """Allow use as dictionary key"""
        return hash(self.bool)

    def __nonzero__(self):
        """Allow truth testing"""
        return self.bool


class GenBooleanError(Exception):
    """Exception class for invalid boolean data"""
    pass

############# Utility Function ############

def _splitFormat(format):
    """Return tuple of format converted to lower case or raise exception"""
    result = format.lower().split('/', 1)
    if len(result) != 2 or not result[0] or not result[1] or \
       result[0] == result[1]:
        raise GenBooleanError('Invalid boolean format')
    return result
