#!/usr/bin/env python

"""
****************************************************************************
 modsitecustom.py, Linux script for TreeLine to modify sitecustomize
                   script for unicode support

 Copyright (C) 2004, Douglas W. Bell

 This is free software; you can redistribute it and/or modify it under the
 terms of the GNU General Public License, Version 2.  This program is
 distributed in the hope that it will be useful, but WITTHOUT ANY WARRANTY.
*****************************************************************************
"""

import sys, os.path, getopt, shutil, py_compile, re, distutils.sysconfig

progName = 'TreeLine'

def usage(exitCode=2):
    """Display usage info and exit"""
    print 'Usage:'
    print '    python modsitecustom.py [-h] [-e ext]'
    print 'where:'
    print '    -h         display this help message'
    print '    -e ext     backup original sitecustomize file with suffix ext'
    sys.exit(exitCode)

def modSiteCustom(siteCustomExt=''):
    """Add lines to the sitecustomize.py file in site-packages"""
    global progName
    lines = ['# added by the %s installer\n' % progName, \
             'import sys\n', \
             'sys.setappdefaultencoding = sys.setdefaultencoding']
    siteDir = distutils.sysconfig.get_python_lib()
    sitePath = os.path.join(siteDir, 'sitecustomize.py')
    if os.access(sitePath, os.R_OK):
        f = open(sitePath, 'r')
        origLines = f.read()
        f.close()
        if origLines.find(lines[-1].split()[0]) >= 0:
            print '  Sitecustomize change already in place.'
            return    # function already saved
        if siteCustomExt:
            try:
                backupPath = sitePath + siteCustomExt
                shutil.copy2(sitePath, backupPath)
                print '  Saved original sitecustomize file to %s' % backupPath
            except IOError, e:
                if str(e).find('Permission denied') >= 0:
                    print '    Warning - must be root to modify '\
                              'sitecustomize file'
                    print '    Skipping sitecustomize change'
                    print '    Unicode may not function properly'
                    return
                raise
    print '  Writing to sitecustomize file'
    try:
        f = open(sitePath, 'a')
        f.writelines(lines)
        f.close()
        py_compile.compile(sitePath)
    except IOError, e:
        if str(e).find('Permission denied') >= 0:
            print '    Warning - must be root to modify sitecustomize file'
            print '    Skipping sitecustomize change'
            print '    Unicode may not function properly'
            return
        raise

def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], 'he:')
    except getopt.GetoptError:
        usage(2)
    siteCustomExt = ''
    for opt, val in opts:
        if opt == '-h':
            usage(0)
        elif opt == '-e':
            siteCustomExt = val
    modSiteCustom(siteCustomExt)


if __name__ == '__main__':
    main()
